/*
 * Created on 16.08.2004
 * Copyright (C) 2004 Sebastian Baltes (sebastian.baltes@gmx.de)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 */

package de.sbaltes.fractal;
import java.awt.Component;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.MemoryImageSource;

/**
 * @author Sebastian Baltes
 */
public class FloodMandelbrotComponent extends Component implements Runnable {

  MemoryImageSource fractalImageSource;
  Image fractalImage;
  MandelbrotCalculator calculator;
  int width;
  int height;
  Frame frame;
  String originalTitle;
  private int n;
  private long t0;
  
  
  public FloodMandelbrotComponent(Frame frame) {
    this.frame=frame;
    this.originalTitle=frame.getTitle();
  }
  
  public void init() {
    this.addComponentListener(new ComponentListener() {
      public void componentHidden(ComponentEvent e) {
      }
      public void componentMoved(ComponentEvent e) {
      }
      public void componentResized(ComponentEvent e) {
        initFractalImage();
      }
      public void componentShown(ComponentEvent e) {
      }});
    this.addMouseListener(new MouseListener() {
      public void mouseClicked(MouseEvent arg0) {
        if (arg0.getButton()==MouseEvent.BUTTON1) {
          calculator.zoom(arg0.getPoint(),4F);
        } else {
          calculator.zoom(arg0.getPoint(),0.25F);
        }
      }
      public void mouseEntered(MouseEvent arg0) {
      }
      public void mouseExited(MouseEvent arg0) {
      }
      public void mousePressed(MouseEvent arg0) {
      }
      public void mouseReleased(MouseEvent arg0) {
      }
    });

    this.requestFocus();
    calculator = new MandelbrotCalculator();
    calculator.initFractalParameters();
    initFractalImage();
    new Thread(this).start();
  }


  public void run() {
    try {
      while (true) {
        render();
      }
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  synchronized void initFractalImage() {
    width = this.getWidth();
    height = this.getHeight();
    calculator.setDimensions(width, height);
    fractalImageSource = new MemoryImageSource(width, height, calculator.fractalPixels, 0, width);
    fractalImageSource.setAnimated(true);
    fractalImage = createImage(fractalImageSource);
    fractalImageSource.setAnimated(true);
  }

  public synchronized void render() {
    if (fractalImage != null) {      
      calculator.calculateOneIteration();
      fractalImageSource.newPixels();
    }
    Graphics g = getGraphics();
    if (g != null) {
      paint(g);
      g.dispose();
    }
    n++;
    long t = System.currentTimeMillis();
    if (t - t0 > 1000) {
      frame.setTitle(originalTitle+" fps=" + ((n * 1000) / (t - t0)));        
      n = 0;
      t0 = t;
    }
  }

  public void paint(Graphics g) {
    if (fractalImage != null) {
      g.drawImage(fractalImage, 0, 0, width, height, null);
    }
  }

}